@@ -14,16 +14,17 @@ random.seed(0)
 
 # http://matplotlib.1069221.n5.nabble.com/How-to-turn-off-matplotlib-DEBUG-msgs-td48822.html
 import matplotlib
-import logging
-mpl_logger = logging.getLogger('matplotlib')
-mpl_logger.setLevel(logging.WARNING)
+# Removed because logging inhibits multiprocessing
+#mpl_logger = logging.getLogger('matplotlib')
+#mpl_logger.setLevel(logging.WARNING)
 
 # Importing all due to large number of attacker types
 from .attackers import *
 from .attackers import Basic_Attacker, Even_Turn_Attacker
 from .api import create_app
 
-from .ddos_simulator import DDOS_Simulator
+
+from .ddos_simulators import DDOS_Simulator, Fluid_DDOS_Simulator
 from .graphers import Animater, Combination_Grapher, Grapher
 
 # Importing all due to large number of manager types
