@@ -9,14 +9,13 @@ __email__ = "jfuruness@gmail.com, agorbenko97@gmail.com"
 __status__ = "Development"
 
 from argparse import ArgumentParser
-from logging import DEBUG
 import os
 from sys import argv
 
 from .api import create_app
-from .ddos_simulator import DDOS_Simulator
-from .managers import Manager
-from .utils import config_logging
+from .ddos_simulators import DDOS_Simulator, Fluid_DDOS_Simulator
+from .managers import Manager, Protag_Manager_No_Merge
+from .utils import Log_Levels
 from .graphers import Combination_Grapher
 
 def main():
@@ -38,40 +37,45 @@ def main():
 
     parser.add_argument("--save", dest="save", default=False, action="store_true")
     parser.add_argument("--high_res", dest="high_res", default=False, action="store_true")
-    parser.add_argument("--trials", type=int, dest="trials", default=100)
+    parser.add_argument("--trials", type=int, dest="trials", default=10)
     parser.add_argument("--graph_dir", type=str, dest="graph_dir", default=os.path.join("/tmp", "lib_ddos_simulator"))
     parser.add_argument("--api", dest="api", default=False, action="store_true")
 
 
     args = parser.parse_args()
-    if args.debug:
-        config_logging(DEBUG)
 
     if args.api:
         create_app().run(debug=True)
     elif args.animate:
-        # NOTE: for optimal animations,
-        # use 24, 4, 8, 10 for users, attackers, buckets, threshold
-        DDOS_Simulator(args.num_users,  # number of users
-                       args.num_attackers,  # number of attackers
-                       args.num_buckets,  # number of buckets
-                       args.threshold,  # Threshold
-                       Manager.runnable_managers,
-                       graph_dir=args.graph_dir,
-                       save=args.save,
-                       high_res=args.high_res).run(args.rounds, animate=True, graph_trials=False)
+        for sim_cls in reversed(DDOS_Simulator.runnable_simulators):
+            # NOTE: for optimal animations,
+            # use 24, 4, 8, 10 for users, attackers, buckets, threshold
+            sim_cls(args.num_users,  # number of users
+                    args.num_attackers,  # number of attackers
+                    args.num_buckets,  # number of buckets
+                    args.threshold,  # Threshold
+                    Manager.runnable_managers,
+                    graph_dir=args.graph_dir,
+                    save=args.save,
+                    stream_level=Log_Levels.DEBUG if args.debug else Log_Levels.INFO,
+                    high_res=args.high_res).run(args.rounds,
+                                                animate=True,
+                                                graph_trials=False)
     elif args.graph_combos:
-        Combination_Grapher(graph_dir=args.graph_dir,
+        Combination_Grapher(stream_level=Log_Levels.DEBUG if args.debug else Log_Levels.INFO,
+                            graph_dir=args.graph_dir,
                             tikz=args.tikz,
                             save=args.save,
                             high_res=args.high_res).run(trials=args.trials)
     else:
-        DDOS_Simulator(args.num_users,
-                       args.num_attackers,
-                       args.num_buckets,
-                       args.threshold,
-                       Manager.runnable_managers,
-                       graph_dir=args.graph_dir,
-                       save=args.save,
-                       tikz=args.tikz,
-                       high_res=args.high_res).run(args.rounds)
+        for sim_cls in DDOS_Simulator.runnable_simulators:
+            sim_cls(args.num_users,
+                    args.num_attackers,
+                    args.num_buckets,
+                    args.threshold,
+                    Manager.runnable_managers,
+                    stream_level=Log_Levels.DEBUG if args.debug else Log_Levels.INFO,
+                    graph_dir=args.graph_dir,
+                    save=args.save,
+                    tikz=args.tikz,
+                    high_res=args.high_res).run(args.rounds)
