@@ -68,11 +68,23 @@ class Animater(Base_Grapher):
                  "detected_location", "blue_to_yellow",
                  "yellow_to_blue", "track_suspicions"]
 
-    def __init__(self, manager, **kwargs):
+    def __init__(self,
+                 manager,
+                 sim_cls,
+                 user_cls,
+                 attacker_cls,
+                 **kwargs):
         """Initializes simulation"""
 
 
         super(Animater, self).__init__(**kwargs)
+
+        self.sim_cls = sim_cls
+        self.user_cls = user_cls
+        self.attacker_cls = attacker_cls
+
+        self.disconnected_location = (-20, -20)
+
         self.low_dpi = 60
         if self.high_res:
             # Anything higher than 600 and you need to drastically increase bitrate
@@ -92,7 +104,7 @@ class Animater(Base_Grapher):
         self.max_users, self.fig, self.ax = self._format_graph()
         #assert self.save or len(manager.users) <= 40,\
         #    "Matplotlib can't handle that many users"
-        self.ogusers = deepcopy(manager.users)
+        self.ogusers = deepcopy(self.users)
 
         fontsize = 12
         matplotlib.rcParams.update({'font.size': 10})
@@ -128,6 +140,7 @@ class Animater(Base_Grapher):
             self.frames_per_round = 100
         self.total_rounds = 0
         self.detected_location = (-10, -10,)
+
         # Frames left before turning non attacked to attacked
         self.percent_left_before_change = .2
 
@@ -154,7 +167,10 @@ class Animater(Base_Grapher):
 
     @property
     def users(self):
-        return self.manager.users + self.manager.eliminated_users
+        will_be_conn = []
+        for user_list in self.manager.will_be_connected_users.values():
+            will_be_conn.extend(user_list)
+        return list(self.manager.users.values()) + will_be_conn
 
     @property
     def buckets(self):
@@ -187,6 +203,15 @@ class Animater(Base_Grapher):
             user.detected = True
             user.suspicions.append(0)
 
+        for user in manager.disconnected_users:
+            user.points.append(self.disconnected_location)
+            user.suspicions.append(0)
+
+        for user_list in manager.will_be_connected_users.values():
+            for user in user_list:
+                user.points.append(self.disconnected_location)
+                user.suspicions.append(0)
+
 
     def run_animation(self, total_rounds):
         """Graphs data
@@ -315,23 +340,25 @@ class Animater(Base_Grapher):
         """Creates patches of users"""
 
         self.user_patches = []
-        for bucket in self.buckets:
-            for user in bucket.users:
-                user.patch = plt.Circle((bucket.patch_center(), 5),
-                                        User.patch_radius,
-                                        fc=user.og_face_color)
-                if isinstance(user, Attacker):
-                    user.horns = plt.Polygon(0 * self.get_horn_array(user),
-                                             fc=user.og_face_color,
-                                             **dict(ec="k"))
-                    user.horns.set_linewidth(.4 if self.high_res else .5)
-
-                user.text = plt.text(bucket.patch_center(),
-                                     5,
-                                     f"{user.id}",
-                                     horizontalalignment='center',
-                                     verticalalignment='center')
-                self.user_patches.append(user.patch)
+        for user in self.users:
+            if user.bucket.patch is None:
+                bucket_patch_center = self.disconnected_location[0]
+            else:
+                bucket_patch_center = user.bucket.patch_center()
+            user.patch = plt.Circle((bucket_patch_center, 5),
+                                    User.patch_radius,
+                                    fc=user.og_face_color)
+            if isinstance(user, Attacker):
+                user.horns = plt.Polygon(0 * self.get_horn_array(user),
+                                         fc=user.og_face_color,
+                                         **dict(ec="k"))
+                user.horns.set_linewidth(.4 if self.high_res else .5)
+            user.text = plt.text(bucket_patch_center,
+                                 5,
+                                 f"{user.id}",
+                                 horizontalalignment='center',
+                                 verticalalignment='center')
+            self.user_patches.append(user.patch)
 
     def init(self):
         """inits the animation
@@ -372,7 +399,7 @@ class Animater(Base_Grapher):
 
         self.round_text = plt.text(self.ax.get_xlim()[1] * .5,
                                    self.ax.get_ylim()[1] - .5,
-                                   f"{self.name}: Round 0",
+                                   self._get_round_text(0),
                                    fontsize=12 if self.high_res else 12,
                                    bbox=round_text_kwargs,
                                    horizontalalignment='center',
@@ -422,12 +449,25 @@ class Animater(Base_Grapher):
                 and i % self.frames_per_round== 0):
                 user.text.set_text("Detected")
                 user.patch.set_facecolor("grey")
+            elif (future_point == self.disconnected_location
+                  and current_point != self.disconnected_location
+                  and i % self.frames_per_round== 0):
+                  user.text.set_text("Disconnected")
+                  user.patch.set_facecolor("purple")
             else:
                 if self.track_suspicions and i % self.frames_per_round == 0:
                     text = f"{user.suspicions[(i//self.frames_per_round) + 1]:.1f}"
                     user.text.set_text(f"{user.id:2.0f}:{text}")
                 if i == 0:
                     user.patch.set_facecolor(user.og_face_color)
+                if (current_point not in [self.disconnected_location, self.detected_location]
+                    and future_point not in [self.disconnected_location, self.detected_location]):
+                    user.patch.set_facecolor(user.og_face_color)
+                    text = user.text.get_text().lower()
+                    # String comparisons no!!!
+                    if "disconnected" in text or "detected" in text:
+                        text = str(user.id)
+                        user.text.set_text(text)
 
     def animate_buckets(self, i):
         for bucket in self.buckets:
@@ -462,8 +502,7 @@ class Animater(Base_Grapher):
         # This is why it works best with that sizing
         self.round_text = plt.text(self.ax.get_xlim()[1] * .5,
                                    self.ax.get_ylim()[1] - .5,
-                                   (f"{self.name}: "
-                                    f"Round {i // self.frames_per_round}"),
+                                   self._get_round_text(i),
                                    fontsize=12 if self.high_res else 12,
                                    bbox=round_text_kwargs,
                                    horizontalalignment='center',
@@ -495,6 +534,14 @@ class Animater(Base_Grapher):
                          ])
         return horn_array
 
+    def _get_round_text(self, round_num):
+        return (f"{self.name}: "
+                f"Round {round_num // self.frames_per_round}     "
+                f"{self.sim_cls.__name__}|||"
+                f"{self.user_cls.__name__}|||"
+                f"{self.attacker_cls.__name__}")
+ 
+
 
 # Basically just makes the colors pretty
 # https://matplotlib.org/3.1.0/gallery/lines_bars_and_markers/gradient_bar.html
