@@ -8,10 +8,12 @@ __maintainer__ = "Justin Furuness"
 __email__ = "jfuruness@gmail.com, agorbenko97@gmail.com"
 __status__ = "Development"
 
+from copy import deepcopy
 import os
 
 import matplotlib.pyplot as plt
 from matplotlib.lines import Line2D
+import matplotlib.patches as mpatches
 from statistics import mean, variance
 from math import sqrt
 from multiprocessing import cpu_count
@@ -22,7 +24,7 @@ from .base_grapher import Base_Grapher
 
 from ..attackers import Attacker
 # Done this way to avoid circular imports
-from .. import ddos_simulator
+from ..ddos_simulators import ddos_simulator
 from ..managers import Manager
 from ..utils import Log_Levels
 
@@ -32,7 +34,6 @@ class Worst_Case_Attacker:
     Later used to graph the worst case attacker graph"""
     pass
 
-
 class Combination_Grapher(Base_Grapher):
     """Compares managers against each other
 
@@ -42,27 +43,38 @@ class Combination_Grapher(Base_Grapher):
 
     __slots__ = ["second_legend"]
 
+    def __init__(self, *args, **kwargs):
+        super(Combination_Grapher, self).__init__(*args, **kwargs)
+        self.second_legend = []
+
     def run(self,
+            ddos_sim_cls_list=None,
             managers=Manager.runnable_managers,
             attackers=Attacker.runnable_attackers,
             num_buckets_list=[10 ** i for i in range(1, 3)],
             users_per_bucket_list=[10 ** i for i in range(1, 4)],
             num_rounds_list=[10 ** i for i in range(1, 4)],
-            trials=50):
+            trials=10):
         """Runs in parallel every possible scenario
 
         Looks complicated, but no real way to simplify it
         so deal with it"""
 
+        if ddos_sim_cls_list is None:
+            ddos_sim_cls_list =\
+                ddos_simulator.DDOS_Simulator.runnable_simulators
+
         # Initializes graph path
         self.make_graph_dir(destroy=True)
 
         # Total number of scenarios
-        pbar_total = (len(num_buckets_list) *
+        pbar_total = (len(ddos_sim_cls_list) *
+                      len(num_buckets_list) *
                       len(users_per_bucket_list) *
                       len(num_rounds_list) *
                       (len(attackers) + 1))  # Add 1 to attacker for worst case
 
+        _pathos_simulators_list = []
         _pathos_num_buckets_list = []
         _pathos_users_per_bucket = []
         _pathos_num_rounds = []
@@ -70,15 +82,18 @@ class Combination_Grapher(Base_Grapher):
             for users_per_bucket in users_per_bucket_list:
                 for num_rounds in num_rounds_list:
                     for attacker in attackers + [Worst_Case_Attacker]:
-                        self.get_attacker_graph_dir(attacker)
+                        for sim_cls in ddos_sim_cls_list:
+                            self.get_attacker_graph_dir(attacker)
 
-                    _pathos_num_buckets_list.append(num_buckets)
-                    _pathos_users_per_bucket.append(users_per_bucket)
-                    _pathos_num_rounds.append(num_rounds)
+                        _pathos_simulators_list.append(sim_cls)
+                        _pathos_num_buckets_list.append(num_buckets)
+                        _pathos_users_per_bucket.append(users_per_bucket)
+                        _pathos_num_rounds.append(num_rounds)
 
         p = ProcessingPool(nodes=cpu_count())
         total = len(_pathos_num_rounds)
-        full_args = [[attackers] * total,
+        full_args = [_pathos_simulators_list,
+                     [attackers] * total,
                      _pathos_num_buckets_list,
                      _pathos_users_per_bucket,
                      _pathos_num_rounds,
@@ -92,12 +107,14 @@ class Combination_Grapher(Base_Grapher):
         if (self.stream_level == Log_Levels.DEBUG
             # https://stackoverflow.com/a/58866220/8903959
             or "PYTEST_CURRENT_TEST" in os.environ):
+
             for i in range(total):
                 try:
                     current_args = [x[i] for x in full_args]
                     self.get_graph_data(*current_args)
                 except Exception as e:
-                    print(current_args)
+                    from pprint import pprint
+                    pprint(current_args)
                     raise e
         else:
             p.map(self.get_graph_data, *full_args)
@@ -108,6 +125,7 @@ class Combination_Grapher(Base_Grapher):
         print()
 
     def get_graph_data(self,
+                       ddos_sim_cls,
                        attackers,
                        num_buckets,
                        users_per_bucket,
@@ -118,6 +136,10 @@ class Combination_Grapher(Base_Grapher):
                        total_num):
         """Gets data for graphing and graphs it"""
 
+        ddos_sim_cls = deepcopy(ddos_sim_cls)
+        attackers = deepcopy(attackers)
+        managers = deepcopy(managers)
+
         scenario_data = {manager: {attacker: {"X": [],
                                               "Y": [],
                                               "YERR": []}
@@ -131,12 +153,15 @@ class Combination_Grapher(Base_Grapher):
             for manager in managers:
                 manager_data = scenario_data[manager][attacker]
                 for percent_attackers in percent_attackers_list:
+                    manager = deepcopy(manager)
+                    attacker = deepcopy(attacker)
                     manager_data["X"].append(percent_attackers)
                     Y = []
                     # TRIALS
                     for _ in range(trials):
                         # Get the utility for each trail and append it
-                        Y.append(self.run_scenario(attacker,
+                        Y.append(self.run_scenario(ddos_sim_cls,
+                                                   attacker,
                                                    num_buckets,
                                                    users_per_bucket,
                                                    num_rounds,
@@ -146,7 +171,8 @@ class Combination_Grapher(Base_Grapher):
                     err_length = 1.645 * 2 * (sqrt(variance(Y))/sqrt(len(Y)))
                     manager_data["YERR"].append(err_length)
 
-            self.graph_scenario(scenario_data,
+            self.graph_scenario(ddos_sim_cls,
+                                scenario_data,
                                 num_buckets,
                                 users_per_bucket,
                                 num_rounds,
@@ -156,7 +182,8 @@ class Combination_Grapher(Base_Grapher):
         worst_case_data = self.worst_case_data(managers,
                                                scenario_data,
                                                attackers)
-        self.graph_scenario(worst_case_data,
+        self.graph_scenario(ddos_sim_cls,
+                            worst_case_data,
                             num_buckets,
                             users_per_bucket,
                             num_rounds,
@@ -164,6 +191,7 @@ class Combination_Grapher(Base_Grapher):
                             write_json=True)
 
     def run_scenario(self,
+                     ddos_sim_cls,
                      attacker,
                      num_buckets,
                      users_per_bucket,
@@ -177,16 +205,16 @@ class Combination_Grapher(Base_Grapher):
         good_users = users - attackers
         # No longer used, but maybe in the future
         threshold = 0
-        sim = ddos_simulator.DDOS_Simulator(good_users,
-                                            attackers,
-                                            num_buckets,
-                                            threshold,
-                                            [manager],
-                                            stream_level=self.stream_level,
-                                            graph_dir=self.graph_dir,
-                                            tikz=self.tikz,
-                                            save=self.save,
-                                            attacker_cls=attacker)
+        sim = ddos_sim_cls(good_users,
+                           attackers,
+                           num_buckets,
+                           threshold,
+                           [manager],
+                           stream_level=self.stream_level,
+                           graph_dir=self.graph_dir,
+                           tikz=self.tikz,
+                           save=self.save,
+                           attacker_cls=attacker)
         # dict of {manager: final utility}
         utilities_dict = sim.run(num_rounds, graph_trials=False)
         return utilities_dict[manager]
@@ -223,6 +251,7 @@ class Combination_Grapher(Base_Grapher):
         return worst_case_scenario_data
 
     def graph_scenario(self,
+                       ddos_sim_cls,
                        scenario_data,
                        num_buckets,
                        users_per_bucket,
@@ -230,7 +259,8 @@ class Combination_Grapher(Base_Grapher):
                        attacker,
                        write_json=False):
 
-        fig, axs, title = self._get_formatted_fig_axs(scenario_data,
+        fig, axs, title = self._get_formatted_fig_axs(ddos_sim_cls,
+                                                      scenario_data,
                                                       num_buckets,
                                                       users_per_bucket,
                                                       num_rounds, attacker)
@@ -253,6 +283,7 @@ class Combination_Grapher(Base_Grapher):
             self.write_json(graph_path, scenario_data)
 
     def _get_formatted_fig_axs(self,
+                               sim_cls,
                                scenario_data,
                                num_buckets,
                                users_per_bucket,
@@ -261,7 +292,8 @@ class Combination_Grapher(Base_Grapher):
         """Creates and formats axes"""
 
         fig, axs = plt.subplots(figsize=(20, 10))
-        title = (f"Scenario: buckets: {num_buckets}, "
+        title = (f"Sim: {sim_cls.__name__}, "
+                 f"Scenario: buckets: {num_buckets}, "
                  f"users: {users_per_bucket * num_buckets}, "
                  f"rounds: {num_rounds}, attacker_cls: {attacker.__name__}")
         fig.suptitle(title)
@@ -309,7 +341,7 @@ class Combination_Grapher(Base_Grapher):
                      label=f"{manager.__name__}",
                      ls=self.styles(manager_i),
                      # https://stackoverflow.com/a/26305286/8903959
-                     marker=None if write_json else self.markers(manager_i))
+                     marker=self.markers(manager_i))
         # This means we are graphing worst case
         if write_json:
             self.overlay_scatter_plot(axs,
@@ -335,7 +367,7 @@ class Combination_Grapher(Base_Grapher):
         axs.scatter(scenario_data[manager][attacker]["X"],
                     scenario_data[manager][attacker]["Y"],
                     c=colors,
-                    s=40,
+                    s=45,
                     zorder=3,
                     marker=self.markers(manager_i))
 
@@ -345,16 +377,7 @@ class Combination_Grapher(Base_Grapher):
             atk_freq_dict[atk] = atk_freq_dict.get(atk, 0) + 1
         atks = list(reversed(sorted(atk_freq_dict, key=atk_freq_dict.get)))
 
-        legend_elements = [Line2D([0],
-                                  [0],
-                                  marker=self.markers(manager_i),
-                                  color=color_dict[atk],
-                                  label=atk,
-                                  markerfacecolor=color_dict[atk],
-                                  markersize=15)
-                           for atk in atks]
-
-        self.second_legend = legend_elements
+        self.second_legend.extend(atks)
 
     def get_worst_case_atk_color_dict(self):
         """Returns a dictionary of attacker to colors"""
@@ -387,13 +410,18 @@ class Combination_Grapher(Base_Grapher):
                            bbox_to_anchor=(1, 0.5))
 
         # If we are adding a second legend for worst case attacker colors
-        if hasattr(self, "second_legend"):
+        if len(self.second_legend) > 0:
+            color_dict = self.get_worst_case_atk_color_dict()
+            legend_elements = [mpatches.Patch(color=color_dict[atk], label=atk)
+                               for atk in set(self.second_legend)]
+
             # https://riptutorial.com/matplotlib/example/32429/multiple-legends-on-the-same-axes
             # https://matplotlib.org/3.1.1/gallery/text_labels_and_annotations/custom_legends.html
-            axs.legend(handles=self.second_legend,
+            axs.legend(handles=legend_elements,
                        loc='upper right',
                        bbox_to_anchor=(1, 1))
             axs.add_artist(first)
+            self.second_legend = []
 
     def write_json(self, graph_path, scenario_data):
         """Writes json file"""
