@@ -10,3 +10,5 @@ __status__ = "Development"
 
 from .bucket import Bucket
 from .user import User
+from .fluid_user import Fluid_User
+from .user_status import User_Status
